@@ -4,24 +4,14 @@ use strict;
 use warnings;
 
 require UR;
-our $VERSION = "0.44_01"; # UR $VERSION;
-
-use Carp qw(croak confess shortmess);
-use constant TRANSACTION_STATE_OPEN => 'open';
-use constant TRANSACTION_STATE_COMMITTED => 'committed';
-
-use Exporter qw(import);
-our @EXPORT_OK = qw(TRANSACTION_STATE_OPEN TRANSACTION_STATE_COMMITTED);
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
     is => ['UR::Context'],
     has => [
-        begin_point     => { is => 'Integer' },
-        end_point       => { is => 'Integer', is_optional => 1},  # FIXME is this ever used anywhere?
-        state           => { is => 'Text', valid_values => [TRANSACTION_STATE_OPEN, TRANSACTION_STATE_COMMITTED] },
-        commit_validator => { default_value => 'changes_can_be_saved',
-                              doc => 'validation function used before commit() can succeed' },
+        begin_point     => {},
+        end_point       => {is_optional => 1},  # FIXME is this ever used anywhere?
+        state           => {}, # open, committed, rolled-back
     ],
     is_transactional => 1,
 );
@@ -33,16 +23,15 @@ our $last_transaction_id = 0;
 
 sub delete {
     my $self = shift;
+    $DB::single = 1;
     $self->rollback;
 }
 
-sub begin {
+sub begin 
+{
     my $class = shift;
-    my %params = @_;
-
-    delete @params{'begin_point', 'end_point', 'state'}; # These are set within this function
-
     my $id = $last_transaction_id++;
+    #my $id = @open_transaction_stack;
 
     my $begin_point = @change_log;
     $log_all_changes = 1;
@@ -56,9 +45,9 @@ sub begin {
     my $self = $class->create(
         id => $id,
         begin_point => $begin_point,
-        state => TRANSACTION_STATE_OPEN,
+        state => "open",
         parent => $last_trans,
-        %params,
+        @_
     );
 
     unless ($self) {
@@ -72,7 +61,8 @@ sub begin {
     return $self;
 }
 
-sub log_change {
+sub log_change
+{
     my $this_class = shift;
     my ($object, $class, $id, $aspect, $undo_data) = @_;
 
@@ -81,7 +71,9 @@ sub log_change {
     # wrappers (create/delete/load/unload/define) signal change also
     # and we undo the wrapper, thereby undoing these
     # -> ignore any signal from a method which is wrapped by another signalling method which gets undone
-    return if ( $aspect eq "load" or
+    return if ($aspect eq "_create_object" or
+               $aspect eq "_delete_object" or
+               $aspect eq "load" or
                $aspect eq "load_external"
               );
 
@@ -89,6 +81,7 @@ sub log_change {
         #print "skipping @_\n";
         return;
     }
+    #print "logging: @_\n";
 
     if ($aspect eq "delete") {
         $undo_data = Data::Dumper::Dumper($object);
@@ -105,7 +98,7 @@ sub log_change {
     );
 
     unless (ref($change)) {
-        #$DB::single = 1;
+        $DB::single = 1;
     }
 
     push @change_log, $change;
@@ -118,7 +111,8 @@ sub has_changes {
     return (@changes > 1 ? 1 : ());
 }
 
-sub get_changes {
+sub get_changes
+{
     my $self = shift;
     my $begin_point = $self->begin_point;
     my $end_point = $self->end_point || $#change_log;
@@ -131,7 +125,8 @@ sub get_changes {
     }
 }
 
-sub get_change_summary {
+sub get_change_summary
+{
     # TODO: This should compress multiple changes to the same object as much as possible
     # Right now, it just omits the creation event for the transaction object itself.
     # -> should the creation of the transaction be part of it?
@@ -147,7 +142,8 @@ sub get_change_summary {
     return @changes;
 }
 
-sub rollback {
+sub rollback
+{
     my $self = shift;
 
     # Support calling as a class method: UR::Context::Transaction->rollback rolls back the current trans
@@ -158,29 +154,18 @@ sub rollback {
         }
     }
 
-    if ($self->state ne TRANSACTION_STATE_OPEN) {
-        Carp::confess("Cannot rollback a transaction that is " . $self->state . ".")
-    }
-
-    $self->__signal_change__('prerollback');
-
     my $begin_point = $self->begin_point;
     unless ($self eq $open_transaction_stack[-1]) {
         # This is not the top transaction on the stack.
         # Rollback internally nested transactions in order from the end.
-        my @transactions_with_begin_point =
-            map { [ $_->begin_point, $_ ] }
+        my @later_transactions =
+            sort { $b->begin_point <=> $a->begin_point }
             $self->class->get(
                 begin_point =>   { operator => ">", value => $begin_point }
             );
-        my @later_transactions =
-            map { $_->[1] }
-            sort { $b->[0] <=> $a->[0] }
-            @transactions_with_begin_point;
-
         for my $later_transaction (@later_transactions) {
             if ($later_transaction->isa("UR::DeletedRef")) {
-                #$DB::single = 1;
+                $DB::single = 1;
             }
             $later_transaction->rollback;
         }
@@ -191,57 +176,37 @@ sub rollback {
         die "Parent transaction $parent is not below this one on the stack $open_transaction_stack[-2]?";
     }
 
-    {
-        # Reverse each change, starting from the most recent, and
-        # ending with the creation of the transaction object itself.
-        local $log_all_changes = 0;
+    # Reverse each change, starting from the most recent, and
+    # ending with the creation of the transaction object itself.
+    local $log_all_changes = 0;
 
 
-        $self->__signal_change__('rollback', 1);
-        my @changes_to_undo = reverse $self->get_changes();
-        my $transaction_change = pop @changes_to_undo;
-        my $transaction = $transaction_change->changed_class_name->get($transaction_change->changed_id);
-        unless ($self == $transaction && $transaction_change->changed_aspect eq 'create') {
-            die "First change was not the creation of this transaction!";
+    my @changes_to_undo = reverse $self->get_changes();
+    for my $change (@changes_to_undo) {
+        if ($change == $changes_to_undo[0]) {
+            # the transaction reverses itself in its own context,
+            # but the removal of the transaction itself happens in the parent context
+            $UR::Context::current = $parent;
         }
-        for my $change (@changes_to_undo) {
-            if ($change == $changes_to_undo[0]) {
-                # the transaction reverses itself in its own context,
-                # but the removal of the transaction itself happens in the parent context
-                $UR::Context::current = $parent;
-            }
-
-            $change->undo;
-            $change->delete;
-        }
-
-        for my $change (@changes_to_undo) {
-            unless($change->isa('UR::DeletedRef')) {
-                Carp::confess("Failed to undo a change during transaction rollback.");
-            }
-        }
-
-        $transaction_change->undo;
-        $transaction_change->delete;
+        $change->undo;
+        $change->delete;
     }
 
     $#change_log = $begin_point-1;
 
     unless($self->isa("UR::DeletedRef")) {
-        #$DB::single = 1;
-        Carp::confess("Failed to remove transaction during rollback.");
+        $DB::single = 1;
+        Carp::confess("Odd number of changes after rollback");
     }
 
     pop @open_transaction_stack;
-    unless (@open_transaction_stack) {
-        $log_all_changes = 0;
-    }
     $UR::Context::current = $parent;
 
     return 1;
 }
 
-sub commit {
+sub commit
+{
     my $self = shift;
 
     # Support calling as a class method: UR::Context::Transaction->commit commits the current transaction.
@@ -252,111 +217,50 @@ sub commit {
         }
     }
 
-    if ($self->state ne TRANSACTION_STATE_OPEN) {
-        Carp::confess("Cannot commit a transaction that is " . $self->state . ".")
+    if ($self->state ne "open") {
+        Carp::confess("Transaction not open!?")
     }
 
     unless ($open_transaction_stack[-1] == $self) {
         # TODO: decide if this should work like rollback, and commit nested transactions automatically
         Carp::confess("Cannot commit a transaction with open sub-transactions!");
     }
-    $self->__signal_change__('precommit');
 
-    my $validator = $self->commit_validator;
-    unless ($self->$validator()) {
-        return;
-    }
-
-    $self->state(TRANSACTION_STATE_COMMITTED);
-    if ($self->state eq TRANSACTION_STATE_COMMITTED) {
-        $self->__signal_change__('commit',1);
-    }
-    else {
-        $self->__signal_change__('commit',0);
-    }
+    $self->state("committed");
     pop @open_transaction_stack;
-    unless (@open_transaction_stack) {
-        $log_all_changes = 0;
-    }
+    #$self->delete();
 
     $UR::Context::current = $self->parent;
     return 1;
 }
 
-sub changes_can_be_saved {
-    my $self = shift;
-
-    # This is very similar to behavior in UR::Context::_sync_databases. The only
-    # reason it isn't re-used from UR::Context is the desire to limit changed
-    # objects to those changed within the transaction.
-    # TODO: limit to objects that changed within transaction as to not duplicate
-    # error checking unnecessarily.
-
-    my @changed_objects =
-        grep { ! $_->isa('UR::DeletedRef') }
-        map  { $_->changed_object() } $self->get_changes();
-
-    # This is primarily to catch custom validity logic in class overrides.
-    my @invalid = grep { $_->__errors__ } @changed_objects;
-    if (@invalid) {
-        $self->display_invalid_data_for_save(\@invalid);
-        return;
+sub execute
+{
+    my $class = shift;
+    Carp::confess("Attempt to call class method on instance.  This is probably not what you want...") if ref $class;
+    my $code = shift;
+    my $transaction = $class->begin;
+    my $result = eval($code->());
+    unless ($result) {
+        $transaction->rollback;
     }
-
-    return 1;
+    if ($@) { 
+        die $@;
+    }
+    $transaction->commit;
+    return $result;
 }
 
-sub eval_or_do {
-    my $is_failure = shift;
-    my $block = shift;
-
-    my $class = __PACKAGE__;
-    if (@_) {
-        confess('%s::eval takes one argument', $class);
-    }
-    my $tx = $class->begin();
-    my $result = CORE::eval { $block->() };
-    my $eval_error = $@;
-
-    if ($is_failure->($result, $eval_error)) {
-        $class->debug_message(shortmess('Rolling back transaction'));
-        $class->debug_message($eval_error) if ($eval_error);
-        unless($tx->rollback()) {
-            Carp::croak 'failed to rollback transaction';
-        }
-    } else {
-        unless($tx->commit()) {
-            Carp::croak 'failed to commit transaction';
-        }
-    }
-
-    if (wantarray) {
-        return ($result, $eval_error);
-    } else {
-        return $result;
-    }
-}
-
-# eval function takes a block (&) sort of like CORE::eval
-# eval will rollback on a caught die
-sub eval(&) {
-    my $is_failure = sub {
-        my ($result, $eval_error) = @_;
-        return $eval_error;
-    };
-    return eval_or_do($is_failure, @_);
-}
-
-# do function takes a block (&) sort of like CORE::do
-# do will rollback on a false result as well as before re-throwing a caught die
-sub do(&) {
-    my $is_failure = sub {
-        my ($result, $eval_error) = @_;
-        return !$result || $eval_error;
-    };
-    my ($result, $eval_error) = eval_or_do($is_failure, @_);
-    if ($eval_error) {
-        croak $eval_error, "\t...propogated";
+sub execute_and_rollback
+{
+    my $class = shift;
+    Carp::confess("Attempt to call class method on instance.  This is probably not what you want...") if ref $class;
+    my $code = shift;
+    my $transaction = $class->begin;
+    my $result = eval($code->());
+    $transaction->rollback;
+    if ($@) {
+        die $@;
     }
     return $result;
 }
@@ -415,16 +319,6 @@ objects.  As all activity to objects occurs in some kind of transaction
 context, the newly created transaction exists within whatever context was
 current before the call to begin().
 
-  $t = UR::Context::Transaction->begin(commit_validator => sub { ... });
-
-A validation function may be assigned with the C<commit_validator> property.
-When the transaction is committed, this function is called.  The commit
-proceeds if this function returns a true value.  The default function,
-C<changes_can_be_saved> requires that all objects changed within the
-transaction be valid, ie. that C<$obj->__errors__()> returns an empty list.
-The validation function is passed one argument: the transaction object
-being committed.
-
 =back
 
 =head1 METHODS
@@ -436,16 +330,7 @@ being committed.
   $t->commit();
 
 Causes all objects with changes to save those changes back to the underlying
-context.
-
-If the validation function (specified with the C<commit_validator> param when
-the transaction was created with C<begin()>) returns false, the changes are
-not committed to the encompassing context, C<commit()> returns false and this
-transaction remains in effect.
-
-Returns true if all the transaction's changes are committed to the encompassing
-Context.  This transaction object then becomes invalid, and its state will be
-'committed'.
+context.  
 
 =item rollback
 
@@ -456,9 +341,6 @@ state when the transaction began.  Classes with properties whose meta-property
 is_transactional => 0 are not tracked within a transaction and will not be
 reverted.
 
-After C<rollback()>, this transaction becomes invalid, and the object will become
-a L<UR::DeletedRef>.
-
 =item delete
 
   $t->delete();
@@ -483,33 +365,31 @@ Return a list or L<UR::Change> objects representing changes within the transacti
 
 =over 4
 
-=item eval
+=item execute
 
-  UR::Context::Transaction::eval BLOCK
+  $retval = UR::Context::Transaction->execute($coderef);
 
-Executes the BLOCK (with no arguments) wrapped by a software transaction and a
-CORE::eval.  If the BLOCK dies then the exception is caught and the software
-transaction is rolled back.
+Executes the coderef with no arguments, within an eval and a software
+transaction.  If the coderef returns true, the transaction is committed.
+If it returns false, the transaction is rolled back.  Finally the coderef's
+return value is returned to the caller.
 
-=item do
+If the coderef throws an exception, it will be caught, the transaction rolled
+back, and the exception will be re-thrown with die().
 
-  UR::Context::Transaction::do BLOCK
+=item execute_and_rollback
 
-Executes the BLOCK (with no arguments) wrapped by a software transaction and a
-CORE::eval.  If the BLOCK returns a true value and does not die then the
-software transaction is committed.  If the BLOCK returns false or dies then the
-software transaction is rolled back.
+  UR::Context::Transaction->execute_and_rollback($coderef);
 
-If the BLOCK throws an exception, it will be caught, the software transaction
-rolled back, and the exception will be re-thrown with die().
+Executes the coderef with no arguments, within an eval and a software
+transaction.  Reguardless of the return value of the coderef, the transaction
+will be rolled back.
+
+If the coderef throws an exception, it will be caught, the transaction rolled
+back, and the exception will be re-thrown with die().
 
 =back
 
-=head1 EXPORTS
-
-This module can export constants that match the valid values of the C<state>
-property: TRANSACTION_STATE_OPEN and TRANSACTION_STATE_COMMITTED
-
 =head1 SEE ALSO
 
 L<UR::Context>
