@@ -7,13 +7,12 @@ use strict;
 use warnings;
 
 use UR;
-our $VERSION = "0.46"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
-    is => 'UR::Namespace::Command::Base',
+    is => 'UR::Namespace::Command',
     has => [
-        classes_or_modules => {
+        subject_list => {
             is_many => 1,
             is_optional => 1,
             shell_args_position => 99
@@ -52,14 +51,18 @@ sub execute
 {
     my $self = shift;
 
+    unless ($self->SUPER::_init(@_)) {
+        return;
+    }
+
     my $params = shift;
 
-    my $namespace = $self->namespace_name;
+    my $namespace = $self->namespace_subdir;
     unless ($namespace) {
         die "This command can only be run from a directory tree under a UR namespace module.\n";
     }
 
-    my @subject_list = $self->classes_or_modules;
+    my @subject_list = $self->subject_list;
 
     if ($self->can("for_each_class_object") ne __PACKAGE__->can("for_each_class_object")) {
 
@@ -139,7 +142,7 @@ sub execute
                 exit 0;
             }
         }
-        #$DB::single = 1;
+        $DB::single = 1;
         while (keys %child_processes) {
             my $child_pid = wait();
             if ($child_pid == -1) {
