@@ -5,29 +5,25 @@ use warnings;
 use above 'UR';
 
 package UR::Namespace::Command::Test::Window;
-require UR;
-our $VERSION = "0.44_01"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
     is => 'Command',
     has => {
-        src => {
+        bare_args => {
             is_optional => 1,
             is_many => 1,
             shell_args_position => 1
         }
-    },
-    doc => 'repl tk window'
+    }
 );
 
 sub execute {
     my $self = shift;
-    require Tk;
     
-    my $src = [$self->src];
-    if (@$src > 0) {
-        for my $code (@$src) {
+    my $bare_args = [$self->bare_args];
+    if (@$bare_args > 0) {
+        for my $code (@$bare_args) {
             no strict;
             no warnings;
             eval $code;
@@ -35,26 +31,28 @@ sub execute {
         }
     }
     else {
-        UR::Namespace::Command::Test::Window::Tk->activate_with_gtk_support;
+        DebugWindow->activate_with_gtk_support;
     }
 
 }
 
-package UR::Namespace::Command::Test::Window::Tk;;
+package DebugWindow;
 
+use Tk;
 our $tmp;
+if ($^O eq 'MSWin32' || $^O eq 'cygwin') {
+    #$tmp = $ENV{TEMP};
+    $tmp ||= 'C:/temp';
+}
+else {
+    $tmp = $ENV{TMPDIR};
+    $tmp ||= '/tmp';
+    require Gtk;
+}
+
 our $workspace;
 
 sub new {
-    if ($^O eq 'MSWin32' || $^O eq 'cygwin') {
-        #$tmp = $ENV{TEMP};
-        $tmp ||= 'C:/temp';
-    }
-    else {
-        $tmp = $ENV{TMPDIR};
-        $tmp ||= '/tmp';
-    }
-
     # Make a window
     my $debug_window  = new MainWindow(-title=>"Debug");
 
@@ -94,7 +92,6 @@ sub new {
 }
 
 sub new_gtk {
-    require Gtk;
     my $debug_window = DebugWindow::new();
     my $frame = $debug_window->Frame()->pack(-expand=>0,-fill=>'both',-anchor=>'nw');
     my $continuous_refresh = 0;
