@@ -2,13 +2,11 @@ package UR::Object::Property::View::DescriptionLineItem::Text;
 
 use strict;
 use warnings;
-require UR;
-our $VERSION = "0.44_01"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
     is => 'UR::Object::View::Default::Text',
-    doc => "View used by 'ur show properties' for each property line item",
+    doc => "View used by 'ur describe' for each property line item",
 );
 
 sub _update_view_from_subject {
@@ -23,19 +21,12 @@ sub _update_view_from_subject {
     unless ($column_name) {
         if ($property_meta->via) {
             $column_name = $property_meta->via . '->' . $property_meta->to;
-        } elsif ($property_meta->is_classwide) {
-            $column_name = '(classwide)';
+        } elsif ($property_meta->is_class_wide) {
+            $column_name = '(class wide)';
             
         } elsif ($property_meta->is_delegated) {
             # delegated, but not via.  Must be an object accessor
             $column_name = ''
-        } elsif ($property_meta->is_calculated) {
-            my $calc_from = $property_meta->calculate_from;
-            if ($calc_from and @$calc_from) {
-                $column_name = '(calculated from ' . join(',',@$calc_from). ')';
-            } else {
-                $column_name = '(calculated)';
-            }
         } else {
             $column_name = '(no column)';
         }
@@ -74,6 +65,6 @@ UR::Object::Property::View::DescriptionLineItem::Text - View class for UR::Objec
 
 =head1 DESCRIPTION
 
-Used by UR::Namespace::Command::Show::Properties when displaying information about a property
+Used by UR::Namespace::Command::Describe when displaying information about a property
 
 =cut
