@@ -1,6 +1,6 @@
-package UR::Object::Tag;
+# Methods for setting transient attributes on objects.
 
-#TODO: update these to be UR::Value objects instead of some ancient hack
+package UR::Object::Tag;
 
 =pod
 
@@ -117,7 +117,7 @@ object.
 require 5.006_000;
 use warnings;
 use strict;
-our $VERSION = "0.44_01"; # UR $VERSION;
+our $VERSION = $UR::VERSION; 
 
 # set up module
 use base qw(UR::ModuleBase);
@@ -142,11 +142,7 @@ UR::Util->generate_readwrite_methods(%default_values);
 sub create($@)
 {
     my ($class, @initial_prop) = @_;
-    my $self = bless({%default_values,@initial_prop},$class);
-    if (not ref($self->{properties}) eq 'ARRAY') {
-        $self->{properties} = [ $self->{properties} ];
-    }
-    return $self;
+    return bless({%default_values,@initial_prop},$class);
 }
 
 sub delete($)
@@ -166,16 +162,6 @@ sub filter
     return $filter;
 }
 
-sub __display_name__ {
-    my $self = shift;
-    my $desc = $self->desc;
-    my $prefix = uc($self->type);
-    my @properties = map { "'$_'" } $self->properties;
-    my $prop_noun = scalar(@properties) > 1 ? 'properties' : 'property';
-    my $msg = "$prefix: $prop_noun " . join(', ', @properties) . ": $desc";
-    return $msg;
-}
-
 1;
 __END__
 
