@@ -2,13 +2,13 @@ package UR::Object::View;
 use warnings;
 use strict;
 require UR;
-our $VERSION = "0.46"; # UR $VERSION;
+our $VERSION = $UR::VERSION;;
 
 class UR::Object::View {
     has_abstract_constant => [
-        subject_class_name      => { is_abstract => 1, is_constant => 1 },#is_classwide => 1, is_constant => 1, is_optional => 0 },
-        perspective             => { is_abstract => 1, is_constant => 1 },#is_classwide => 1, is_constant => 1, is_optional => 0 },
-        toolkit                 => { is_abstract => 1, is_constant => 1 },#is_classwide => 1, is_constant => 1, is_optional => 0 },
+        subject_class_name      => { is_abstract => 1, is_constant => 1 },#is_class_wide => 1, is_constant => 1, is_optional => 0 },           
+        perspective             => { is_abstract => 1, is_constant => 1 },#is_class_wide => 1, is_constant => 1, is_optional => 0 },   
+        toolkit                 => { is_abstract => 1, is_constant => 1 },#is_class_wide => 1, is_constant => 1, is_optional => 0 },
     ],
     has_optional => [
         parent_view => {
@@ -33,20 +33,20 @@ class UR::Object::View {
             is => 'ARRAY',
             is_abstract => 1,
             is_constant => 1,
-            is_many => 1, # technically this is one "ARRAY"
-            default_value => undef,
-            doc => 'a tree of default aspect descriptions' },
+            is_many => 1,
+            default_value => [],
+            doc => 'List of default aspect names' },
     ],
     has_optional_transient => [
         _widget  => { 
+
             doc => 'the object native to the specified toolkit which does the actual visualization' 
         },
         _observer_data => {
-            is => 'HASH',
             is_transient => 1,
-            value => undef, # hashref set at construction time
+            value => {},
             doc => '  hooks around the subject which monitor it for changes'
-        },
+        }
     ],
     has_many_optional => [
         aspect_names    => { via => 'aspects', to => 'name' },
@@ -55,10 +55,10 @@ class UR::Object::View {
 
 
 sub create {
-    my $class = shift;
+    my $class = shift;    
 
     my ($params,@extra) = $class->define_boolexpr(@_);
-
+    
     # set values not specified in the params which can be inferred from the class name
     my ($expected_class,$expected_perspective,$expected_toolkit) = ($class =~ /^(.*)::View::(.*?)::([^\:]+)$/);
     unless ($params->specifies_value_for('subject_class_name')) {
@@ -75,17 +75,16 @@ sub create {
     # now go the other way, and use both to infer a final class name
     $expected_class = $class->_resolve_view_class_for_params($params);
     unless ($expected_class) {
-        my $subject_class_name = $params->value_for('subject_class_name');
-        Carp::croak("Failed to resolve a subclass of " . __PACKAGE__ 
-                . " for $subject_class_name from parameters.  "
-                . "Received $params.");
+        Carp::croak("Failed to resolve a subclass for " . __PACKAGE__ 
+                    . " from parameters.  Expected subject_class_name, perspective,"
+                    . " and toolkit to be part of the parameters, or class definition.  "
+                    . "Received $params.");
     }
 
     unless ($class->isa($expected_class)) {
         return $expected_class->create(@_);
     }
 
-    $params->add_filter(_observer_data => {});
     my $self = $expected_class->SUPER::create($params);
     return unless $self;
 
@@ -129,21 +128,21 @@ sub _resolve_view_class_for_params {
     # The subject must be explicitly of class "SubjectClassName" or some subclass of it.
     my $class = shift;
     my $bx = $class->define_boolexpr(@_);
-
+    
     if (exists $view_class_cache{$bx->id}) {
         if (!defined $view_class_cache{$bx->id}) {
             return;
         }
         return $view_class_cache{$bx->id};
     }
-
+    
     my %params = $bx->params_list;
 
     my $subject_class_name = delete $params{subject_class_name};
     my $perspective = delete $params{perspective};
     my $toolkit = delete $params{toolkit};
     my $aspects = delete $params{aspects};
-
+   
     unless($subject_class_name and $perspective and $toolkit) {
         Carp::confess("Bad params @_.  Expected subject_class_name, perspective, toolkit.");
     }
@@ -157,7 +156,7 @@ sub _resolve_view_class_for_params {
 
     my $subject_class_object = $subject_class_name->__meta__;
     my @possible_subject_class_names = ($subject_class_name,$subject_class_name->inheritance);
-
+    
     my $subclass_name;
     for my $possible_subject_class_name (@possible_subject_class_names) {
 
@@ -177,16 +176,12 @@ sub _resolve_view_class_for_params {
                 )
             )
         );
-
-        my $subclass_meta;
-        eval {
-            $subclass_meta = $subclass_name->__meta__;
-        };
-        if ($@ or not $subclass_meta) {
-            #not a class... keep looking
+    
+        my $subclass_meta = UR::Object::Type->get($subclass_name);
+        unless ($subclass_meta) {
             next;
         }
-
+        
         unless($subclass_name->isa(__PACKAGE__)) {
             Carp::carp("Subclass $subclass_name exists but is not a view?!");
             next;
@@ -274,11 +269,10 @@ sub _subject_is_used_in_an_encompassing_view {
 
 sub all_subject_classes {
     my $self = shift;
-    my @classes = ();
 
-    # suppress error callbacks inside this method
+    my @classes = ();
     my $old_cb = UR::ModuleBase->message_callback('error');
-    UR::ModuleBase->message_callback('error', sub {}) if ($old_cb);
+    UR::ModuleBase->message_callback('error', sub {});
 
     for my $aspect ($self->aspects) {
         unless ($aspect->delegate_view) {
@@ -290,7 +284,7 @@ sub all_subject_classes {
             push @classes, $aspect->delegate_view->all_subject_classes
         }
     }
-    UR::ModuleBase->message_callback('error', $old_cb) if ($old_cb);
+    UR::ModuleBase->message_callback('error', $old_cb);
 
     push @classes, $self->subject_class_name;
 
@@ -307,9 +301,8 @@ sub all_subject_classes_ancestry {
 
     my @aclasses;
     for my $class (@classes) {
-        my $m;
-        eval { $m = $class->__meta__ };
-        next if $@ or not $m;
+        my $m = UR::Object::Type->get($class);
+        next unless $m;
 
         push @aclasses, reverse($class, $m->ancestry_class_names);
     }
@@ -351,11 +344,6 @@ sub _bind_subject {
     return unless defined $subject;
 
     my $observer_data = $self->_observer_data;
-    unless ($observer_data) {
-        $self->_observer_data({});
-        $observer_data = $self->_observer_data;
-    }
-    Carp::confess unless $self->_observer_data == $observer_data;
 
     # See if we've already done this.    
     return 1 if $observer_data->{$subject};
@@ -379,21 +367,21 @@ sub _bind_subject {
     
     # Set the view to show initial data.
     $self->_update_view_from_subject;
-   
+    
     return 1;
 }
 
 sub _update_view_from_subject {
     # This is called whenever the view changes, or the subject changes.
     # It passes the change(s) along, so that the update can be targeted, if the developer chooses.
-    Carp::croak("The _update_view_from_subject method must be implemented for all concreate "
-        . " view subclasses.  No _update_subject_from_view for " 
+    Carp::confess("The _update_view_from_subject method must be implemented for all concreate "
+        . " view subclasses.  No _update_subject_from_viewfor " 
         . (ref($_[0]) ? ref($_[0]) : $_[0]) . "!");
 }
 
 sub _update_subject_from_view {
-    Carp::croak("The _update_subject_from_view method must be implemented for all concreate "
-        . " view subclasses.  No _update_subject_from_view for " 
+    Carp::confess("The _update_subject_from_view method must be implemented for all concreate "
+        . " view subclasses.  No _update_subject_from_viewfor " 
         . (ref($_[0]) ? ref($_[0]) : $_[0]) . "!");
 }
 
