@@ -4,7 +4,7 @@ use warnings;
 use strict;
 require UR;
 
-our $VERSION = "0.46"; # UR $VERSION;
+our $VERSION = $UR::VERSION;;
 
 class UR::Object::View::Aspect {
     id_by => [
@@ -145,18 +145,59 @@ no warnings;
 
     my $name = $self->name;
     my $subject_class_name = $parent_view->subject_class_name;
+    my $property_meta = $subject_class_name->__meta__->property($name);
+
 
     my $retval;
-    
-    my $property_meta = $subject_class_name->__meta__->property($name);
-    my $aspect_type;
     if ($property_meta) {
-        $aspect_type = $property_meta->_data_type_as_class_name;
+        my $aspect_type = $property_meta->data_type;
+
         unless ($aspect_type) {
-            Carp::confess("Undefined aspect type. Set 'is' for $name in class " . $property_meta->class_name);
+            if (my $delegated_to_meta = $property_meta->final_property_meta) {
+                $aspect_type = $delegated_to_meta->data_type;
+            }
         }
 
-        unless ($aspect_type->can("__meta__")) {
+        unless ($aspect_type) {
+            Carp::confess("Property meta for class ".$property_meta->class_name." property ".$property_meta->property_name." has no data_type");
+        }
+
+        if ($aspect_type->can("__meta__")) {
+            my $aspect_meta = $aspect_type->__meta__;
+            
+            my $delegate_view ||= $aspect_type->create_view(
+                subject_class_name => $aspect_type,
+                perspective => $parent_view->perspective,
+                toolkit => $parent_view->toolkit,
+                parent_view => $parent_view,
+                aspects => [],
+            );
+            unless ($delegate_view) {
+                $self->error_message("Error creating delegate view for $name ($aspect_type)!");
+                $self->delete;
+                #return;
+            }
+            my @default_aspects_params = $delegate_view->_resolve_default_aspects();
+           
+            # add aspects which do not "go backward"
+            # no one wants to see an order, with a list of line items, which re-reprsent thier order on each
+            for my $aspect_params (@default_aspects_params) {
+                my $aspect_param_name = (ref($aspect_params) ?  $aspect_params->{name} : $aspect_params);
+                my $aspect_property_meta = $aspect_meta->property($aspect_param_name);
+                no strict; no warnings;
+                if ($aspect_property_meta->reverse_as() eq $name) {
+                    
+                }
+                elsif ($property_meta->reverse_as eq $aspect_param_name) {
+                }
+                else {
+                    $delegate_view->add_aspect(ref($aspect_params) ? %$aspect_params : $aspect_params);
+                }
+            }
+            $self->delegate_view($delegate_view);
+            $retval = $delegate_view;
+        }
+        else {
             Carp::croak("$aspect_type has no meta data?  cannot generate a view for $subject_class_name $name!"); 
         }
     }
@@ -166,66 +207,9 @@ no warnings;
             $self->delete;
             Carp::croak($self->error_message);
         }
-        $aspect_type = 'UR::Value::Text'
+        Carp::croak("property $name on $subject_class_name has no meta?");
     }
 
-    my $aspect_meta = $aspect_type->__meta__;
-
-    my $delegate_view;
-    local $@;
-    eval {
-        $delegate_view = $aspect_type->create_view(
-            subject_class_name => $aspect_type,
-            perspective => $parent_view->perspective,
-            toolkit => $parent_view->toolkit,
-            parent_view => $parent_view,
-            aspects => [],
-        );
-    };
-
-    unless ($delegate_view) {
-        # try again using the "default" perspective
-        my $err1 = $@; 
-        eval {
-            $delegate_view = $aspect_type->create_view(
-                subject_class_name => $aspect_type,
-                perspective => 'default', 
-                toolkit => $parent_view->toolkit,
-                parent_view => $parent_view,
-                aspects => [],
-            );
-        };
-        my $err2 = $@; 
-
-        unless ($delegate_view) {
-            $self->error_message(
-                "Error creating delegate view for $name ($aspect_type)! $err1\n"
-                . "Also failed to fall back to the default perspective for $name ($aspect_type)! $err2"
-            );
-            return;
-        }
-    }
-
-    my @default_aspects_params = $delegate_view->_resolve_default_aspects();
-        
-    # add aspects which do not "go backward"
-    # no one wants to see an order, with a list of line items, which re-reprsent thier order on each
-    for my $aspect_params (@default_aspects_params) {
-        my $aspect_param_name = (ref($aspect_params) ?  $aspect_params->{name} : $aspect_params);
-        my $aspect_property_meta = $aspect_meta->property($aspect_param_name);
-        no strict; no warnings;
-        next if (!$aspect_property_meta or !$property_meta);
-
-        if ($aspect_property_meta->reverse_as ne $name
-            and
-            $property_meta->reverse_as ne $aspect_param_name
-        ) {
-            $delegate_view->add_aspect(ref($aspect_params) ? %$aspect_params : $aspect_params);
-        }
-    }
-    $self->delegate_view($delegate_view);
-    $retval = $delegate_view;
-
     return $retval;
 }
 
