@@ -2,8 +2,6 @@ package UR::Object::View::Default::Json;
 
 use strict;
 use warnings;
-require UR;
-our $VERSION = "0.46"; # UR $VERSION;
 
 use JSON;
 
@@ -12,8 +10,7 @@ class UR::Object::View::Default::Json {
     has_constant => [
         toolkit     => { value => 'json' },
     ],
-    has_optional => [
-        encode_options => { is => 'ARRAY', default_value => ['ascii', 'pretty', 'allow_nonref', 'canonical'], doc => 'Options to enable on the JSON object; see the documentation for the JSON Perl module' },
+    has => [
     ],
 };
 
@@ -21,23 +18,12 @@ my $json;
 sub _json {
     my ($self) = @_;
     return $json if defined $json;
-
-    $json = JSON->new;
-    foreach my $opt ( @{ $self->encode_options } ) {
-        local $@;
-        eval { $json = $json->$opt; };
-        if ($@) {
-            Carp::croak("Can't initialize JSON object for encoding.  Calling method $opt from encode_options died: $@");
-        }
-        if (!$json) {
-            Carp::croak("Can't initialize JSON object for encoding.  Calling method $opt from encode_options returned false");
-        }
-    }
-    return $json;
+    return $json = JSON->new->ascii->pretty->allow_nonref;
 }
 
 sub _generate_content {
     my $self = shift;
+
     return $self->_json->encode($self->_jsobj);
 }
 
@@ -48,10 +34,14 @@ sub _jsobj {
     return '' unless $subject;
 
     my %jsobj = ();
-
-    for my $aspect ($self->aspects) { 
-        my $val = $self->_generate_content_for_aspect($aspect);
-        $jsobj{$aspect->name} = $val if defined $val;
+    
+    unless ($self->_subject_is_used_in_an_encompassing_view()) {
+        # the content for any given aspect is handled separately
+        for my $aspect ($self->aspects) { 
+                
+            my $val = $self->_generate_content_for_aspect($aspect);
+            $jsobj{$aspect->name} = $val if defined $val;
+        }
     }
 
     return \%jsobj;
@@ -65,10 +55,8 @@ sub _generate_content_for_aspect {
     my $aspect_name = $aspect->name;
 
     my $aspect_meta = $self->subject_class_name->__meta__->property($aspect_name);
-    #warn $aspect_name if ref($subject) =~ /Set/;
 
     my @value;
-    local $@;
     eval {
         @value = $subject->$aspect_name;
     };
@@ -76,44 +64,52 @@ sub _generate_content_for_aspect {
         warn $@;
         return;
     }
-
-    # Always look for a delegate view.
-    # This means we replace the value(s) with their
-    # subordinate widget content.
-    unless ($aspect->delegate_view) {
-        $aspect->generate_delegate_view;
+    
+    if (@value == 0) {
+        return; 
+    }
+        
+    if (Scalar::Util::blessed($value[0])) {
+        unless ($aspect->delegate_view) {
+            eval {
+                $aspect->generate_delegate_view;
+            };
+            if ($@) {
+                warn $@;
+            }
+        }
     }
 
     my $ref = [];
-
+ 
+    # Delegate to a subordinate view if needed.
+    # This means we replace the value(s) with their
+    # subordinate widget content.
     if (my $delegate_view = $aspect->delegate_view) {
         foreach my $value ( @value ) {
-            if (Scalar::Util::blessed($value)) {
-                $delegate_view->subject($value);
-            } else {
-                $delegate_view->subject_id($value);
-            }
+            $delegate_view->subject($value);
             $delegate_view->_update_view_from_subject();
-
+            
             if ($delegate_view->can('_jsobj')) {
                 push @$ref, $delegate_view->_jsobj;
             } else {
                 my $delegate_text = $delegate_view->content();
+
                 push @$ref, $delegate_text;
-            }
+            }            
         }
     }
     else {
         for my $value (@value) {
             if (ref($value)) {
-                push @$ref, 'ref';  #TODO(ec) make this render references
+                push @$ref, 'ref';
             } else {
                 push @$ref, $value;
             }
         }
     }
 
-    if ($aspect_meta && $aspect_meta->is_many) {
+    if ($aspect_meta->is_many) {
         return $ref;
     } else {
         return shift @$ref;
