@@ -4,7 +4,6 @@ use UR;
 
 use strict;
 use warnings;
-our $VERSION = "0.46"; # UR $VERSION;
 
 class UR::Service::RPC::Executer {
     has => [
@@ -44,7 +43,7 @@ sub create {
 
 # sub classes can override this
 # If they're going to reject the request, $msg should be modified in place
-# with a return value and exception, because we're going to return it right back
+# with a return value and exception, because we'lre going to return it right back
 # to the requester
 sub authenticate {
 #    my($self,$msg) = @_;
@@ -78,25 +77,18 @@ sub execute {
 
 
         my $method_name = join('::',$target_class, $method);
-        if (! $target_class->can($method)) {
-            $resp_msg_args{exception} =
-                qq(Can't locate object method "$method" via package "$target_class" (perhaps you forgot to load "$target_class"?));
-
+        if ($wantarray) {
+            my @retval;
+            eval { no strict 'refs'; @retval = &{$method_name}(@arglist); };
+            $resp_msg_args{'return_values'} = \@retval unless ($@);
+        } elsif (defined $wantarray) {
+            my $retval;
+            eval { no strict 'refs'; no warnings; $retval = &{$method_name}(@arglist); };
+            $resp_msg_args{'return_values'} = [$retval] unless ($@);
         } else {
-            local $@;
-            if ($wantarray) {
-                my @retval;
-                eval { no strict 'refs'; @retval = &{$method_name}(@arglist); };
-                $resp_msg_args{return_values} = \@retval unless ($@);
-            } elsif (defined $wantarray) {
-                my $retval;
-                eval { no strict 'refs'; no warnings; $retval = &{$method_name}(@arglist); };
-                $resp_msg_args{return_values} = [$retval] unless ($@);
-            } else {
-                eval { no strict 'refs'; &{$method_name}(@arglist); };
-            }
-            $resp_msg_args{exception} = $@ if $@;
+            eval { no strict 'refs'; &{$method_name}(@arglist); };
         }
+        $resp_msg_args{'exception'} = $@ if $@;
         $response = UR::Service::RPC::Message->create(%resp_msg_args);
 
     } else {
@@ -182,7 +174,7 @@ Called when the Server detects data is available to read on its file handle.
 It deserializes the message and calls authenticate.  If authentication fails,
 it immediately passes the message object back to the caller.
 
-If authentication succeeds, it calls the appropriate method in the Executor
+If authentication succeedes, it calls the appropriate method in the Executor
 package, and creates a new Message object with the return value to pass back
 to the caller.
 
