@@ -4,11 +4,11 @@ use strict;
 use warnings;
 
 require UR;
-our $VERSION = "0.44_01"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => 'UR::Value::FOF',
     is => ['UR::Value'],
+    english_name => 'blob',
 );
 
 1;
